@@ -1,4 +1,4 @@
-# Copyright 2013, Sean B. Palmer
+# Copyright 2013-4, Sean B. Palmer
 # Source: http://inamidst.com/saxo/
 
 import atexit
@@ -59,8 +59,10 @@ def parse(octets):
     params = regex_parameter.findall(text[match_prefix.end():])
     return match_prefix.groups(), params[0], params[1:]
 
-class ThreadSafeEnvironment(object):
-    def __init__(self, saxo, prefix, command, parameters):
+class Message(object):
+    def __init__(self, saxo, octets):
+        prefix, command, parameters = parse(octets)
+
         self.base = saxo.base[:]
         self.config = saxo.config_cache.copy()
 
@@ -73,16 +75,7 @@ class ThreadSafeEnvironment(object):
 
         self.command = command
         self.parameters = parameters
-
-        if command == "PRIVMSG":
-            self.sender = self.parameters[0]
-            self.text = self.parameters[1]
-            self.private = self.sender == self.config["nick"]
-            if self.private:
-                self.sender = self.nick
-            if saxo.address:
-                # TODO: Why was this 498 for duxlot?
-                self.limit = 493 - len(self.sender + saxo.address)
+        self.blocked = False
 
         def send(*args):
             saxo.send(*args)
@@ -92,16 +85,45 @@ class ThreadSafeEnvironment(object):
             saxo.send("PRIVMSG", *args)
         self.msg = msg
 
-        if hasattr(self, "sender"):
+        if self.command == "PRIVMSG":
+            self.sender = self.parameters[0]
+            self.text = self.parameters[1]
+            self.private = self.sender == self.config["nick"]
+
+            if self.private:
+                self.sender = self.nick
+
+            if saxo.address:
+                # TODO: Why was this 498 for duxlot?
+                self.limit = 493 - len(self.sender + saxo.address)
+
             def say(text):
                 saxo.send("PRIVMSG", self.sender, text)
             self.say = say
 
-        if hasattr(self, "nick") and hasattr(self, "sender"):
             def reply(text):
                 saxo.send("PRIVMSG", self.sender, self.nick + ": " + text)
             self.reply = reply
 
+            private = self.config.get("private")
+            if self.private:
+                if private == "deny":
+                    self.blocked = True
+            elif private == "only":
+                self.blocked = True
+
+            self.cmd = None
+
+            cmdpfx = self.config["prefix"]
+            if self.text.startswith(cmdpfx):
+                text = self.text[len(cmdpfx):]
+                if " " in text:
+                    cmd, arg = text.split(" ", 1)
+                else:
+                    cmd, arg = text, ""
+                self.cmd = cmd
+                self.arg = arg
+
     def client(self, *args):
         incoming.put(args)
 
@@ -173,10 +195,12 @@ class Saxo(object):
 
         self.config_cache = {}
         client_options = {
-            "channels", # Channels to join on startup
+            "channels", # Channels to join on startup
             "nick", # Nickname of the bot, variable
             "owner", # Full address of the owner
-            "prefix" # Command prefix
+            "prefix", # Command prefix
+            "flood", # Whether or not to flood
+            "private" # Whether to respond in private
         }
 
         for option in opt["client"]:
@@ -288,9 +312,13 @@ class Saxo(object):
     def cancel_discotimer(self):
         if self.discotimer is not None:
             try:
+                was_finished = self.discotimer.finished.is_set()
+                # Note: .cancel() still works on an expired timer!
+                # It just calls .finished.set()
                 self.discotimer.cancel()
+                if not was_finished:
+                    debug("Cancelled probably unfinished disco timer")
                 self.discotimer = None
-                debug("Cancelled the disco timer")
             except:
                 ...
 
@@ -328,18 +356,23 @@ class Saxo(object):
         # We rely on self.disconnect() to have done the right thing
         # The following is a check to make sure that it has
         if self.socket_threads_active():
-           time.sleep(2)
-           if self.socket_threads_active():
-               # TODO: If the threads are still active, they should be killed
-               # Unfortunately, threads in python can't be killed
-               debug("ERROR! Unable to stop the socket threads")
-               os._exit(1)
+            # Wait up to six seconds for the threads to quit
+            for attempt in range(12):
+                time.sleep(0.5)
+                if not self.socket_threads_active():
+                    break
+            else:
+                # TODO: If the threads are still active, they should be killed
+                # Unfortunately, threads in python can't be killed
+                debug("ERROR! Unable to stop the socket threads")
+                os._exit(1)
 
         if "flood" not in self.opt["client"]:
             time.sleep(3)
 
         try: self.connect()
         except SaxoConnectionError as err:
+            # Retry, with 1sec more delay, up to a maximum of 30sec delay
             def connect():
                 incoming.put(("connect", min(30, delay + 1)))
             t = threading.Timer(delay, connect)
@@ -347,9 +380,9 @@ class Saxo(object):
 
     def instruction_connected(self):
         if ":connected" in self.events:
-            irc = ThreadSafeEnvironment(self, ("", "", ""), "", [])
+            msg = Message(self, b"NOOP")
             for function in self.events[":connected"]:
-                function(irc)
+                function(msg)
 
     def instruction_command(self, prefix, sender, cmd, arg):
         self.command(prefix, sender, cmd, arg)
@@ -443,11 +476,10 @@ class Saxo(object):
     def instruction_prefix(self, pfx):
         self.update_config("client", "prefix", pfx)
 
-    def instruction_propagate(self, kind="both"):
-        ...
-
     def instruction_quit(self):
         # Never call this from a thread, otherwise this can give an OSError
+        # TODO: Get the sender to pick this up and disconnect from there?
+        # Could be a problem if the sender has broken
         self.send("QUIT")
         self.disconnect()
         # sys.exit()
@@ -460,11 +492,13 @@ class Saxo(object):
         # TODO: Unit test for :connected event
         incoming.put(("connected",))
 
-    def instruction_reconnect(self, wait=3):
+    def instruction_reconnect(self):
         # disco_* will automatically reconnect
         if self.receiving or self.receiving_thread.is_alive():
+            # Close the socket, which forces the receiving thread to quit
             self.disconnect()
         elif self.sending or self.sending_thread.is_alive():
+            # Signal the sending thread to quit
             outgoing.put(None)
         else:
             debug("Reconnect requested during reconnect")
@@ -479,56 +513,31 @@ class Saxo(object):
 
     def instruction_remote(self, octets):
         debug(repr(octets))
-        nick = self.opt["client"]["nick"]
-        prefix, command, parameters = parse(octets)
+        msg = Message(self, octets)
 
-        if command == "PRIVMSG":
-            private = self.opt["client"].get("private")
-            sender = parameters[0]
-
-            if sender == nick:
-                if private == "deny":
-                    return
-                sender = prefix[0]
-            elif private == "only":
+        if msg.command == "PRIVMSG":
+            if msg.blocked:
                 return
 
-            pfx = self.opt["client"]["prefix"]
-            length = len(pfx)
-            privmsg = parameters[1]
+            if msg.cmd is not None:
+                self.command(msg)
 
-            if privmsg.startswith(pfx):
-                privmsg = privmsg[length:]
-                if " " in privmsg:
-                    cmd, arg = privmsg.split(" ", 1)
-                else:
-                    cmd, arg = privmsg, ""
-
-                self.command(prefix, sender, cmd, arg)
-
-        elif command == "PONG":
+        elif msg.command == "PONG":
             self.cancel_discotimer()
 
-        args = (self, prefix, command, parameters)
-        irc = ThreadSafeEnvironment(*args)
-        def safe(function, irc):
-            try: function(irc)
-            except Exception as err:
-                debug("Thread error:", function.__name__, err)
-
-        def run(name, safe, irc):
+        def run(name, msg):
             if name in self.events:
                 for function in self.events[name]:
-                    if not function.saxo_synchronous:
-                        common.thread(safe, function, irc)
-                    else:
-                        safe(function, irc)
+                    try: function(msg)
+                    except Exception as err:
+                        debug("Error:", function.__name__ + ":", err)
 
         if self.first is True:
-            run(":1st", safe, irc)
+            run(":1st", msg)
             self.first = False
-        run(command, safe, irc)
-        run("*", safe, irc)
+
+        run(msg.command, msg)
+        run("*", msg)
 
     def instruction_schedule(self, unixtime, command, args):
         command = command.encode("ascii")
@@ -541,7 +550,9 @@ class Saxo(object):
     def instruction_sending(self):
         self.sending = True
 
-    def command(self, prefix, sender, cmd, arg):
+    def command(self, msg):
+        cmd, arg = msg.cmd, msg.arg
+
         if ("\x00" in cmd) or (os.sep in cmd) or ("." in cmd):
             return
 
@@ -575,14 +586,14 @@ class Saxo(object):
                     outs = "Sorry, %s responded with an error" % cmd
 
             if outs:
-                self.send("PRIVMSG", sender, outs)
+                self.send("PRIVMSG", msg.sender, outs)
 
         if os.path.isfile(path):
             env = self.environment_cache.copy()
-            env["SAXO_NICK"] = prefix[0]
-            env["SAXO_SENDER"] = sender
-            if sender in self.links:
-                env["SAXO_URL"] = self.links[sender]
+            env["SAXO_NICK"] = msg.nick
+            env["SAXO_SENDER"] = msg.sender
+            if msg.sender in self.links:
+                env["SAXO_URL"] = self.links[msg.sender]
             common.thread(process, env, path, arg)
 
     def update_config(self, section, option, value):
