@@ -1,10 +1,6 @@
-# Copyright 2013, Sean B. Palmer
+# Copyright 2013-4, Sean B. Palmer
 # Source: http://inamidst.com/saxo/
 
-version = "0.2.006"
-# WARNING: If updating anything before this message,
-# change the offset in setup.py
-
 # TODO list:
 # - Think about folding common.py into saxo.py
 # - Allow wildcards in [client].owner
@@ -22,18 +18,16 @@ version = "0.2.006"
 # - Document the database tables
 # - Delete and select methods for sqlite.Table
 
+import os
+
 path = None
 
 if "__file__" in vars():
     if __file__:
-        import os.path
-
         path = os.path.abspath(__file__)
         path = os.path.realpath(path)
         path = os.path.dirname(path)
 
-        del os
-
 if "__path__" in vars():
     if __path__:
         for directory in __path__:
@@ -44,10 +38,25 @@ if "__path__" in vars():
 
         del directory
 
+if path is None:
+   raise Exception("Can't create saxo.path")
+
+with open(os.path.join(path, "version")) as f:
+    version = f.read().rstrip("\r\n")
+
+from collections import namedtuple
+
+version_info = namedtuple(
+    "Version", ["major", "minor", "serial"]
+)(*(int(n) for n in version.split(".")))
+
+del namedtuple
+del os
+
 # Decorators:
 #
 # saxo.command(name)
-# saxo.event(command="*", synchronous=False)
+# saxo.event(command="*")
 # saxo.pipe(function)
 # saxo.setup(function)
 #
@@ -82,7 +91,7 @@ def client(command, *args, base=None):
 def command(name, owner=False):
     def is_owner(irc):
         import re
-        config_owner = irc.config["owner"]
+        config_owner = irc.config.get("owner", "")
         test_identity = False
         if not "!" in config_owner:
             test_identity = True
@@ -150,10 +159,9 @@ def env(name, alternative=None):
     return os.environ.get("SAXO_%s" % name.upper(), alternative)
 
 # TODO: priority?
-def event(command="*", synchronous=False):
+def event(command="*"):
     def decorate(function):
         function.saxo_event = command
-        function.saxo_synchronous = synchronous
         return function
     return decorate
 
@@ -191,7 +199,6 @@ def pipe(function):
 
     try: result = function(arg)
     except Exception as err:
-        import os.path
         import traceback
 
         python = err.__class__.__name__ + ": " + str(err)
